@@ -18,33 +18,10 @@
 
 
 #include "goxel.h"
+#include <errno.h>
 
 #define VERSION 2 // Current version of the file format.
 
-#ifndef NO_ZLIB
-#   include <zlib.h>
-#else
-
-// If we don't have zlib, we simulate the gz functions using normal file
-// operations.
-typedef FILE *gzFile;
-static gzFile gzopen(const char *path, const char *mode) {
-    return fopen(path, mode);
-}
-static void gzclose(gzFile file) {fclose(file);}
-static size_t gzread(gzFile file, void *buff, size_t size) {
-    return fread(buff, size, 1, file);
-}
-static size_t gzwrite(gzFile file, const void *buff, size_t size) {
-    return fwrite(buff, size, 1, file);
-}
-static int gzeof(gzFile file) {return feof(file);}
-static int gzseek(gzFile file, long offset, int whence) {
-    return fseek(file, offset, whence);
-}
-
-#endif
-
 /*
  * File format, version 2:
  *
@@ -54,8 +31,8 @@ static int gzseek(gzFile file, long offset, int whence) {
  *  4 bytes magic string        : "GOX "
  *  4 bytes version             : 2
  *  List of chunks:
- *      4 bytes: data length
  *      4 bytes: type
+ *      4 bytes: data length
  *      n bytes: data
  *      4 bytes: CRC
  *
@@ -115,51 +92,58 @@ typedef struct {
     int      pos;
 } chunk_t;
 
-static void write_int32(gzFile out, int32_t v)
+static void write_int32(FILE *out, int32_t v)
 {
-    gzwrite(out, (char*)&v, 4);
+    fwrite((char*)&v, 4, 1, out);
 }
 
-static int32_t read_int32(gzFile in)
+static int32_t read_int32(FILE *in)
 {
     int32_t v;
-    gzread(in, &v, 4);
+    if (fread(&v, 4, 1, in) != 1) {
+        // XXX: use a better error mechanism!
+        LOG_E("Error reading file");
+    }
     return v;
 }
 
-static bool chunk_read_start(chunk_t *c, gzFile in)
+static bool chunk_read_start(chunk_t *c, FILE *in)
 {
+    size_t r;
     memset(c, 0, sizeof(*c));
-    gzread(in, c->type, 4);
-    if (gzeof(in)) return false;
+    r = fread(c->type, 4, 1, in);
+    if (r == 0) return false; // eof.
+    if (r != 1) LOG_E("Error reading file");
     c->length = read_int32(in);
     return true;
 }
 
-static void chunk_read(chunk_t *c, gzFile in, char *buff, int size)
+static void chunk_read(chunk_t *c, FILE *in, char *buff, int size)
 {
     c->pos += size;
     assert(c->pos <= c->length);
-    if (buff)
-        gzread(in, buff, size);
-    else
-        gzseek(in, size, SEEK_CUR);
+    if (buff) {
+        // XXX: use a better error mechanism!
+        if (fread(buff, size, 1, in) != 1) LOG_E("Error reading file");
+    } else {
+        fseek(in, size, SEEK_CUR);
+    }
 }
 
-static int32_t chunk_read_int32(chunk_t *c, gzFile in)
+static int32_t chunk_read_int32(chunk_t *c, FILE *in)
 {
     int32_t v;
     chunk_read(c, in, (char*)&v, 4);
     return v;
 }
 
-static void chunk_read_finish(chunk_t *c, gzFile in)
+static void chunk_read_finish(chunk_t *c, FILE *in)
 {
     assert(c->pos == c->length);
     read_int32(in); // TODO: check crc.
 }
 
-static bool chunk_read_dict_value(chunk_t *c, gzFile in,
+static bool chunk_read_dict_value(chunk_t *c, FILE *in,
                                   char *key, char *value, int *value_size) {
     int size;
     assert(c->pos <= c->length);
@@ -176,7 +160,7 @@ static bool chunk_read_dict_value(chunk_t *c, gzFile in,
     return true;
 }
 
-static void chunk_write_start(chunk_t *c, gzFile out, const char *type)
+static void chunk_write_start(chunk_t *c, FILE *out, const char *type)
 {
     memset(c, 0, sizeof(*c));
     assert(strlen(type) == 4);
@@ -184,7 +168,7 @@ static void chunk_write_start(chunk_t *c, gzFile out, const char *type)
     c->buffer = calloc(1, CHUNK_BUFF_SIZE);
 }
 
-static void chunk_write(chunk_t *c, gzFile out, const char *data, int size)
+static void chunk_write(chunk_t *c, FILE *out, const char *data, int size)
 {
     if (size == 0) return;
     assert(c->length + size < CHUNK_BUFF_SIZE);
@@ -192,12 +176,12 @@ static void chunk_write(chunk_t *c, gzFile out, const char *data, int size)
     c->length += size;
 }
 
-static void chunk_write_int32(chunk_t *c, gzFile out, int32_t v)
+static void chunk_write_int32(chunk_t *c, FILE *out, int32_t v)
 {
     chunk_write(c, out, (char*)&v, 4);
 }
 
-static void chunk_write_dict_value(chunk_t *c, gzFile out, const char *name,
+static void chunk_write_dict_value(chunk_t *c, FILE *out, const char *name,
                                    const void *data, int size)
 {
     chunk_write_int32(c, out, (int)strlen(name));
@@ -206,22 +190,22 @@ static void chunk_write_dict_value(chunk_t *c, gzFile out, const char *name,
     chunk_write(c, out, data, size);
 }
 
-static void chunk_write_finish(chunk_t *c, gzFile out)
+static void chunk_write_finish(chunk_t *c, FILE *out)
 {
-    gzwrite(out, c->type, 4);
+    fwrite(c->type, 4, 1, out);
     write_int32(out, c->length);
-    gzwrite(out, c->buffer, c->length);
+    fwrite(c->buffer, c->length, 1, out);
     write_int32(out, 0);        // CRC XXX: todo.
     free(c->buffer);
     c->buffer = NULL;
 }
 
-static void chunk_write_all(gzFile out, const char *type,
+static void chunk_write_all(FILE *out, const char *type,
                             const char *data, int size)
 {
-    gzwrite(out, type, 4);
+    fwrite(type, 4, 1, out);
     write_int32(out, size);
-    gzwrite(out, data, size);
+    fwrite(data, size, 1, out);
     write_int32(out, 0);        // CRC XXX: todo.
 }
 
@@ -234,24 +218,24 @@ void save_to_file(const char *path, bool with_preview)
     chunk_t c;
     int nb_blocks, index, size, bpos[3];
     uint64_t uid;
-    gzFile out;
+    FILE *out;
     uint8_t *png, *preview;
     camera_t *camera;
     mesh_iterator_t iter;
 
-    out = gzopen(path, str_endswith(path, ".gz") ? "wb" : "wbT");
+    out = fopen(path, "wb");
     if (!out) {
-        LOG_E("Cannot save to %s", path);
+        LOG_E("Cannot save to %s: %s", path, strerror(errno));
         return;
     }
-    gzwrite(out, "GOX ", 4);
+    fwrite("GOX ", 4, 1, out);
     write_int32(out, VERSION);
 
     // Write image info.
     chunk_write_start(&c, out, "IMG ");
-    if (!box_is_null(goxel->image->box))
-        chunk_write_dict_value(&c, out, "box", &goxel->image->box,
-                               sizeof(goxel->image->box));
+    if (!box_is_null(goxel.image->box))
+        chunk_write_dict_value(&c, out, "box", &goxel.image->box,
+                               sizeof(goxel.image->box));
     chunk_write_finish(&c, out);
 
     if (with_preview) {
@@ -265,7 +249,7 @@ void save_to_file(const char *path, bool with_preview)
 
     // Add all the blocks data into the hash table.
     index = 0;
-    DL_FOREACH(goxel->image->layers, layer) {
+    DL_FOREACH(goxel.image->layers, layer) {
         iter = mesh_get_iterator(layer->mesh, MESH_ITER_BLOCKS);
         while (mesh_iter(&iter, bpos)) {
             mesh_get_block_data(layer->mesh, &iter, bpos, &uid);
@@ -288,7 +272,7 @@ void save_to_file(const char *path, bool with_preview)
     }
 
     // Write all the layers.
-    DL_FOREACH(goxel->image->layers, layer) {
+    DL_FOREACH(goxel.image->layers, layer) {
         chunk_write_start(&c, out, "LAYR");
         nb_blocks = 0;
         if (!layer->base_id) {
@@ -331,7 +315,7 @@ void save_to_file(const char *path, bool with_preview)
     }
 
     // Write all the cameras.
-    DL_FOREACH(goxel->image->cameras, camera) {
+    DL_FOREACH(goxel.image->cameras, camera) {
         chunk_write_start(&c, out, "CAMR");
         chunk_write_dict_value(&c, out, "name", camera->name,
                                strlen(camera->name));
@@ -343,7 +327,7 @@ void save_to_file(const char *path, bool with_preview)
                                sizeof(camera->ofs));
         chunk_write_dict_value(&c, out, "ortho", &camera->ortho,
                                sizeof(camera->ortho));
-        if (camera == goxel->image->active_camera)
+        if (camera == goxel.image->active_camera)
             chunk_write_dict_value(&c, out, "active", NULL, 0);
 
         chunk_write_finish(&c, out);
@@ -354,7 +338,7 @@ void save_to_file(const char *path, bool with_preview)
         free(data);
     }
 
-    gzclose(out);
+    fclose(out);
 }
 
 // Iter info of a gox file, without actually reading it.
@@ -364,15 +348,15 @@ int gox_iter_infos(const char *path,
                                    void *value, void *user),
                    void *user)
 {
-    gzFile in;
+    FILE *in;
     chunk_t c;
     uint8_t *png;
     char magic[4];
 
-    in = gzopen(path, "rb");
+    in = fopen(path, "rb");
 
-    gzread(in, magic, 4);
-    assert(strncmp(magic, "GOX ", 4) == 0);
+    if (fread(magic, 4, 1, in) != 1) return -1;
+    if (strncmp(magic, "GOX ", 4) != 0) return -1;
     read_int32(in);
 
     while (chunk_read_start(&c, in)) {
@@ -389,7 +373,7 @@ int gox_iter_infos(const char *path,
         }
         chunk_read_finish(&c, in);
     }
-    gzclose(in);
+    fclose(in);
     return 0;
 }
 
@@ -407,7 +391,7 @@ int load_from_file(const char *path)
 {
     layer_t *layer, *layer_tmp;
     block_hash_t *blocks_table = NULL, *data, *data_tmp;
-    gzFile in;
+    FILE *in;
     char magic[4] = {};
     uint8_t *voxel_data;
     int nb_blocks;
@@ -421,10 +405,10 @@ int load_from_file(const char *path)
     uint64_t uid = 1;
     camera_t *camera;
 
-    in = gzopen(path, "rb");
+    in = fopen(path, "rb");
     if (!in) return -1;
 
-    gzread(in, magic, 4);
+    if (fread(magic, 4, 1, in) != 1) goto error;
     if (strncmp(magic, "GOX ", 4) != 0) goto error;
     version = read_int32(in);
     if (version > VERSION) {
@@ -434,12 +418,12 @@ int load_from_file(const char *path)
 
     // Remove all layers.
     // XXX: we should load the image fully before deleting the current one.
-    DL_FOREACH_SAFE(goxel->image->layers, layer, layer_tmp) {
+    DL_FOREACH_SAFE(goxel.image->layers, layer, layer_tmp) {
         mesh_delete(layer->mesh);
         free(layer);
     }
-    goxel->image->layers = NULL;
-    memset(&goxel->image->box, 0, sizeof(goxel->image->box));
+    goxel.image->layers = NULL;
+    memset(&goxel.image->box, 0, sizeof(goxel.image->box));
 
     while (chunk_read_start(&c, in)) {
         if (strncmp(c.type, "BL16", 4) == 0) {
@@ -457,7 +441,7 @@ int load_from_file(const char *path)
             free(png);
 
         } else if (strncmp(c.type, "LAYR", 4) == 0) {
-            layer = image_add_layer(goxel->image);
+            layer = image_add_layer(goxel.image);
             nb_blocks = chunk_read_int32(&c, in);   assert(nb_blocks >= 0);
             for (i = 0; i < nb_blocks; i++) {
                 index = chunk_read_int32(&c, in);   assert(index >= 0);
@@ -495,11 +479,11 @@ int load_from_file(const char *path)
             }
         } else if (strncmp(c.type, "CAMR", 4) == 0) {
             camera = camera_new("unnamed");
-            DL_APPEND(goxel->image->cameras, camera);
+            DL_APPEND(goxel.image->cameras, camera);
             while ((chunk_read_dict_value(&c, in, dict_key, dict_value,
                                           &dict_value_size))) {
                 if (strcmp(dict_key, "name") == 0)
-                    sprintf(camera->name, "%s", dict_value);
+                    strncpy(camera->name, dict_value, sizeof(camera->name));
                 if (strcmp(dict_key, "dist") == 0)
                     memcpy(&camera->dist, dict_value, dict_value_size);
                 if (strcmp(dict_key, "rot") == 0)
@@ -509,8 +493,8 @@ int load_from_file(const char *path)
                 if (strcmp(dict_key, "ortho") == 0)
                     memcpy(&camera->ortho, dict_value, dict_value_size);
                 if (strcmp(dict_key, "active") == 0) {
-                    goxel->image->active_camera = camera;
-                    camera_set(&goxel->camera, camera);
+                    goxel.image->active_camera = camera;
+                    camera_set(&goxel.camera, camera);
                 }
             }
 
@@ -518,7 +502,7 @@ int load_from_file(const char *path)
             while ((chunk_read_dict_value(&c, in, dict_key, dict_value,
                                           &dict_value_size))) {
                 if (strcmp(dict_key, "box") == 0)
-                    memcpy(&goxel->image->box, dict_value, dict_value_size);
+                    memcpy(&goxel.image->box, dict_value, dict_value_size);
             }
         } else {
             // Ignore other blocks.
@@ -535,20 +519,21 @@ int load_from_file(const char *path)
         free(data);
     }
 
-    goxel->image->path = strdup(path);
+    goxel.image->path = strdup(path);
+    goxel.image->saved_key = image_get_key(goxel.image);
     goxel_update_meshes(-1);
-    gzclose(in);
+    fclose(in);
 
     // Update plane, snap mask and camera pos not to confuse people.
-    plane_from_vectors(goxel->plane, goxel->image->box[3],
+    plane_from_vectors(goxel.plane, goxel.image->box[3],
                        VEC(1, 0, 0), VEC(0, 1, 0));
-    if (box_is_null(goxel->image->box)) goxel->snap_mask |= SNAP_PLANE;
-    camera_fit_box(&goxel->camera, goxel->image->box);
+    if (box_is_null(goxel.image->box)) goxel.snap_mask |= SNAP_PLANE;
+    camera_fit_box(&goxel.camera, goxel.image->box);
 
     return 0;
 
 error:
-    gzclose(in);
+    fclose(in);
     return -1;
 }
 
@@ -559,8 +544,8 @@ static void action_open(const char *path)
         path = noc_file_dialog_open(NOC_FILE_DIALOG_OPEN, "gox\0*.gox\0",
                                     NULL, NULL);
     if (!path) return;
-    image_delete(goxel->image);
-    goxel->image = image_new();
+    image_delete(goxel.image);
+    goxel.image = image_new();
     load_from_file(path);
 }
 
@@ -578,11 +563,12 @@ static void save_as(const char *path, bool with_preview)
                                     NULL, "untitled.gox");
         if (!path) return;
     }
-    if (path != goxel->image->path) {
-        free(goxel->image->path);
-        goxel->image->path = strdup(path);
+    if (path != goxel.image->path) {
+        free(goxel.image->path);
+        goxel.image->path = strdup(path);
+        goxel.image->saved_key = image_get_key(goxel.image);
     }
-    save_to_file(goxel->image->path, with_preview);
+    save_to_file(goxel.image->path, with_preview);
 }
 
 ACTION_REGISTER(save_as,
@@ -593,7 +579,7 @@ ACTION_REGISTER(save_as,
 
 static void save(const char *path, bool with_preview)
 {
-    save_as(path ?: goxel->image->path, with_preview);
+    save_as(path ?: goxel.image->path, with_preview);
 }
 
 ACTION_REGISTER(save,
