@@ -21,7 +21,7 @@
 static void export_as_txt(const char *path)
 {
     FILE *out;
-    mesh_t *mesh = goxel->layers_mesh;
+    mesh_t *mesh = goxel.layers_mesh;
     int p[3];
     uint8_t v[4];
     mesh_iterator_t iter;
