@@ -120,11 +120,11 @@ static int vxl_import(const char *path)
         }
     }
 
-    mesh_blit(goxel->image->active_layer->mesh, (uint8_t*)cube,
+    mesh_blit(goxel.image->active_layer->mesh, (uint8_t*)cube,
               -w / 2, -h / 2, -d / 2, w, h, d, NULL);
     goxel_update_meshes(-1);
-    if (box_is_null(goxel->image->box)) {
-        bbox_from_extents(goxel->image->box, vec3_zero, w / 2, h / 2, d / 2);
+    if (box_is_null(goxel.image->box)) {
+        bbox_from_extents(goxel.image->box, vec3_zero, w / 2, h / 2, d / 2);
     }
     free(cube);
     free(data);
@@ -242,7 +242,7 @@ static void export_as_vxl(const char *path)
 {
     uint8_t (*map)[512][512][64];
     uint32_t (*color)[512][512][64];
-    const mesh_t *mesh = goxel->layers_mesh;
+    const mesh_t *mesh = goxel.layers_mesh;
     mesh_iterator_t iter = {0};
     uint8_t c[4];
     int x, y, z, pos[3];
@@ -252,6 +252,7 @@ static void export_as_vxl(const char *path)
 
     path = path ?: noc_file_dialog_open(NOC_FILE_DIALOG_SAVE,
                     "vxl\0*.vxl\0", NULL, "untitled.vxl");
+    if (!path) return;
 
     for (z = 0; z < 64; z++)
     for (y = 0; y < 512; y++)
