@@ -44,8 +44,10 @@
 #include <string.h>
 #include <stdarg.h>
 
-#define GOXEL_VERSION_STR "0.8.0"
-#define GOXEL_DEFAULT_THEME "original"
+#define GOXEL_VERSION_STR "0.8.1"
+#ifndef GOXEL_DEFAULT_THEME
+#   define GOXEL_DEFAULT_THEME "original"
+#endif
 
 // #### Set the DEBUG macro ####
 #ifndef DEBUG
@@ -126,6 +128,9 @@ enum {
 
 #ifdef __EMSCRIPTEN__
 #   include <emscripten.h>
+#   define KEEPALIVE EMSCRIPTEN_KEEPALIVE
+#else
+#   define KEEPALIVE
 #endif
 
 // #### Include OpenGL #########
@@ -487,7 +492,7 @@ int b64_decode(const char *src, void *dest);
  * Return:
  *   The new crc64 value.
  */
-uint64_t crc64(uint64_t crc, const uint8_t *s, uint64_t len);
+uint64_t crc64(uint64_t crc, const void *s, uint64_t len);
 
 // #############################
 
@@ -548,6 +553,9 @@ int gl_gen_fbo(int w, int h, GLenum format, int msaa,
 typedef struct {
     void *user;
     void (*set_window_title)(void *user, const char *title);
+    const char *(*get_user_dir)(void *user);
+    const char *(*get_clipboard_text)(void* user);
+    void (*set_clipboard_text)(void *user, const char *text);
 } sys_callbacks_t;
 extern sys_callbacks_t sys_callbacks;
 
@@ -576,6 +584,12 @@ int sys_list_dir(const char *dir,
                                  void *user),
                  void *user);
 
+/*
+ * Function: sys_delete_file
+ * Delete a file from the system.
+ */
+int sys_delete_file(const char *path);
+
 /*
  * Function: sys_get_user_dir
  * Return the user config directory for goxel
@@ -775,28 +789,33 @@ enum {
     ICON_VISIBILITY_OFF = 26,
     ICON_ARROW_DOWNWARD = 27,
     ICON_ARROW_UPWARD = 28,
+    ICON_EDIT = 29,
+    ICON_COPY = 30,
+    ICON_GALLERY = 31,
+    ICON_INFO = 32,
 
-    ICON_TOOLS = 33,
-    ICON_PALETTE = 34,
-    ICON_LAYERS = 35,
-    ICON_RENDER = 36,
-    ICON_CAMERA = 37,
-    ICON_IMAGE = 38,
-    ICON_EXPORT = 39,
-    ICON_DEBUG = 40,
+    ICON_SETTINGS = 33,
+    ICON_CLOUD = 34,
 
-    ICON_VIEW = 41,
-    ICON_MATERIAL = 42,
+    ICON_TOOLS = 41,
+    ICON_PALETTE = 42,
+    ICON_LAYERS = 43,
+    ICON_RENDER = 44,
+    ICON_CAMERA = 45,
+    ICON_IMAGE = 46,
+    ICON_EXPORT = 47,
+    ICON_DEBUG = 48,
+
+    ICON_VIEW = 49,
+    ICON_MATERIAL = 50,
 };
 
-
 /*
- * Some icons have there color blended depending on the style.  We define
+ * Some icons have their color blended depending on the style.  We define
  * them with a range in the icons atlas:
  */
 #define ICON_COLORIZABLE_START 17
-#define ICON_COLORIZABLE_END   33
-
+#define ICON_COLORIZABLE_END   41
 
 // #### Tool/Operation/Painter #
 
@@ -992,13 +1011,13 @@ int mesh_generate_vertices(const mesh_t *mesh, const int block_pos[3],
 // XXX: use int[2][3] for the box?
 void mesh_crop(mesh_t *mesh, const float box[4][4]);
 
-/* Function: mesh_crc32
- * Compute the crc32 of the mesh data as an array of xyz rgba values.
+/* Function: mesh_crc64
+ * Compute the crc64 of the mesh data as an array of xyz rgba values.
  *
  * This is only used in the tests, to make sure that we can still open
  * old file formats.
  */
-uint32_t mesh_crc32(const mesh_t *mesh);
+uint64_t mesh_crc64(const mesh_t *mesh);
 
 // #### Renderer ###############
 
@@ -1271,6 +1290,8 @@ typedef struct inputs
     int         framebuffer; // Screen framebuffer
 } inputs_t;
 
+// Conveniance function to add a char in the inputs.
+void inputs_insert_char(inputs_t *inputs, uint32_t c);
 
 
 /* #########################################
@@ -1456,6 +1477,12 @@ void camera_get_ray(const camera_t *camera, const float win[2],
  */
 void camera_fit_box(camera_t *camera, const float box[4][4]);
 
+/*
+ * Function: camera_get_key
+ * Return a value that is guarantied to change when the camera change.
+ */
+uint64_t camera_get_key(const camera_t *camera);
+
 typedef struct history history_t;
 
 typedef struct layer layer_t;
@@ -1483,9 +1510,10 @@ struct image {
     float    box[4][4];
 
     // For saving.
-    char    *path;
-    int     export_width;
-    int     export_height;
+    char     *path;
+    int      export_width;
+    int      export_height;
+    uint64_t saved_key;     // image_get_key() value of saved file.
 
     image_t *history;
     image_t *history_next, *history_prev;
@@ -1503,6 +1531,12 @@ void image_undo(image_t *img);
 void image_redo(image_t *img);
 bool image_layer_can_edit(const image_t *img, const layer_t *layer);
 
+/*
+ * Function: image_get_key
+ * Return a value that is guarantied to change when the image change.
+ */
+uint64_t image_get_key(const image_t *img);
+
 // ##### Procedural rendering ########################
 
 // The possible states of the procedural program.
@@ -1530,13 +1564,7 @@ int proc_parse(const char *txt, gox_proc_t *proc);
 void proc_release(gox_proc_t *proc);
 int proc_start(gox_proc_t *proc, const float box[4][4]);
 int proc_stop(gox_proc_t *proc);
-int proc_iter(gox_proc_t *proc);
-
-// Get the list of programs saved in data/procs.
-int proc_list_examples(void (*f)(int index,
-                                 const char *name, const char *code,
-                                 void *user), void *user);
-
+int proc_iter(gox_proc_t *proc, mesh_t *mesh, const painter_t *painter);
 
 // Represent a 3d cursor.
 // The program keeps track of two cursors, that are then used by the tools.
@@ -1644,6 +1672,7 @@ typedef struct goxel
     uint8_t    back_color[4];
     uint8_t    grid_color[4];
     uint8_t    image_box_color[4];
+    bool       hide_box;
 
     texture_t  *pick_fbo;
     painter_t  painter;
@@ -1667,8 +1696,6 @@ typedef struct goxel
         float  camera_ofs[3];
     } move_origin;
 
-    gox_proc_t proc;        // The current procedural rendering (if any).
-
     palette_t  *palettes;   // The list of all the palettes
     palette_t  *palette;    // The current color palette
     char       *help_text;  // Seen in the bottom of the screen.
@@ -1678,12 +1705,14 @@ typedef struct goxel
     double     frame_time;  // Clock time at beginning of the frame (sec)
     double     fps;         // Average fps.
     bool       quit;        // Set to true to quit the application.
+    bool       show_wireframe; // Show debug wireframe on meshes.
 
     struct {
         gesture_t drag;
         gesture_t pan;
         gesture_t rotate;
         gesture_t hover;
+        gesture_t pinch;
     } gestures;
 
     // Hold info about the cycles rendering task.
@@ -1696,10 +1725,13 @@ typedef struct goxel
         bool force_restart;
     } render_task;
 
+    // Used to check if the active mesh changed to play tick sound.
+    uint64_t    last_mesh_key;
+    double      last_click_time;
 } goxel_t;
 
 // the global goxel instance.
-extern goxel_t *goxel;
+extern goxel_t goxel;
 
 // XXX: add some doc.
 void goxel_init(void);
@@ -1712,7 +1744,8 @@ void goxel_render_export_view(const float viewport[4]);
 // Called by the gui when the mouse hover a 3D view.
 // XXX: change the name since we also call it when the mouse get out of
 // the view.
-void goxel_mouse_in_view(const float viewport[4], const inputs_t *inputs);
+void goxel_mouse_in_view(const float viewport[4], const inputs_t *inputs,
+                         bool capture_keys);
 
 // Recompute the meshes.  mask from MESH_ enum.
 void goxel_update_meshes(int mask);
@@ -1754,6 +1787,7 @@ void gui_render(void);
 bool gui_collapsing_header(const char *label);
 void gui_text(const char *label, ...);
 bool gui_button(const char *label, float w, int icon);
+bool gui_button_right(const char *label, int icon);
 void gui_group_begin(const char *label);
 void gui_group_end(void);
 bool gui_checkbox(const char *label, bool *v, const char *hint);
@@ -1779,6 +1813,9 @@ float gui_get_avail_width(void);
 void gui_same_line(void);
 void gui_enabled_begin(bool enabled);
 void gui_enabled_end(void);
+// Add an icon in top left corner of last item.
+void gui_floating_icon(int icon);
+
 void gui_alert(const char *title, const char *msg);
 
 void gui_columns(int count);
@@ -1861,9 +1898,12 @@ void *cache_get(cache_t *cache, const void *key, int keylen);
 
 // ####### Sound #################################
 void sound_init(void);
-void sound_play(const char *sound);
+void sound_play(const char *sound, float volume, float pitch);
 void sound_iter(void);
 
+bool sound_is_enabled(void);
+void sound_set_enabled(bool v);
+
 // Section: cycles
 void cycles_init(void);
 void cycles_release(void);
