@@ -28,8 +28,19 @@ extern "C" {
             w = f[3] / 255.; }     \
 
 #define IMGUI_DEFINE_MATH_OPERATORS
-#include "imgui.h"
-#include "imgui_internal.h"
+#define IMGUI_DISABLE_OBSOLETE_FUNCTIONS
+#define IMGUI_INCLUDE_IMGUI_USER_INL
+
+#include "../ext_src/imgui/imgui.h"
+#include "../ext_src/imgui/imgui_internal.h"
+
+#ifndef GUI_TOOLS_COLUMNS_NB
+#   define GUI_TOOLS_COLUMNS_NB 4
+#endif
+
+#ifndef GUI_PALETTE_COLUMNS_NB
+#   define GUI_PALETTE_COLUMNS_NB 8
+#endif
 
 extern "C" {
     bool gui_settings_popup(void *data);
@@ -51,6 +62,7 @@ namespace ImGui {
     void GoxBox(ImVec2 pos, ImVec2 size, bool selected,
                 int rounding_corners_flags = ~0);
     void GoxBox2(ImVec2 pos, ImVec2 size, ImVec4 color, bool fill,
+                 float thickness = 1,
                  int rounding_corners_flags = ~0);
 };
 
@@ -115,7 +127,6 @@ typedef struct gui_t {
     int     current_panel;
     view_t  view;
     int     min_panel_size;
-    bool    use_cycles;
     struct {
         gesture_t drag;
         gesture_t hover;
@@ -127,18 +138,20 @@ typedef struct gui_t {
     struct {
         const char *title;
         bool      (*func)(void *data);
+        void      (*on_closed)(void);
         int         flags;
-        void       *data;
-        bool       opened;
-    } popup;
+        void       *data; // Automatically released when popup close.
+        bool        opened;
+    } popup[8]; // Stack of modal popups
+    int popup_count;
 } gui_t;
 
 static gui_t *gui = NULL;
 
 // Notify the gui that we want the panel size to be at least as large as
 // the last item.
-static void auto_adjust_panel_size(void) {
-    float w = ImGui::GetItemRectMax().x;
+static void auto_adjust_panel_size(float w = 0) {
+    if (w == 0) w = ImGui::GetItemRectMax().x;
     ImGuiStyle& style = ImGui::GetStyle();
     if (ImGui::GetScrollMaxY() > 0) w += style.ScrollbarSize;
     gui->min_panel_size = max(gui->min_panel_size, w);
@@ -146,7 +159,7 @@ static void auto_adjust_panel_size(void) {
 
 static void on_click(void) {
     if (DEFINED(GUI_SOUND))
-        sound_play("click");
+        sound_play("click", 1.0, 1.0);
 }
 
 static bool isCharPressed(int c)
@@ -426,7 +439,7 @@ void gui_release(void)
 }
 
 // XXX: Move this somewhere else.
-void render_axis_arrows(goxel_t *goxel, const float view_size[2])
+void render_axis_arrows(const float view_size[2])
 {
     const float AXIS[][3] = {{1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
     int i;
@@ -436,9 +449,9 @@ void render_axis_arrows(goxel_t *goxel, const float view_size[2])
     uint8_t c[4];
     float s = 1;
     float view[4] = {0, 0, view_size[0], view_size[1]};
-    camera_get_ray(&goxel->camera, spos, view, pos, normal);
-    if (goxel->camera.ortho)
-        s = goxel->camera.dist / 32;
+    camera_get_ray(&goxel.camera, spos, view, pos, normal);
+    if (goxel.camera.ortho)
+        s = goxel.camera.dist / 32;
     else
         vec3_iaddk(pos, normal, 100);
 
@@ -447,7 +460,7 @@ void render_axis_arrows(goxel_t *goxel, const float view_size[2])
         vec4_set(c, AXIS[i][0] * 255,
                     AXIS[i][1] * 255,
                     AXIS[i][2] * 255, 255);
-        render_line(&goxel->rend, pos, b, c);
+        render_line(&goxel.rend, pos, b, c);
     }
 }
 
@@ -459,7 +472,7 @@ void render_view(const ImDrawList* parent_list, const ImDrawCmd* cmd)
     const float height = ImGui::GetIO().DisplaySize.y;
     // XXX: 8 here means 'export' panel.  Need to use an enum or find a
     // better way!
-    if (gui->current_panel == 8 && goxel->render_task.status) {
+    if (gui->current_panel == 8 && goxel.render_task.status) {
         goxel_render_export_view(view->rect);
     } else {
         goxel_render_view(view->rect);
@@ -500,7 +513,7 @@ static void tools_panel(void)
 
     gui_group_begin(NULL);
     for (i = 0; i < nb; i++) {
-        v = goxel->tool->id == values[i].tool;
+        v = goxel.tool->id == values[i].tool;
         sprintf(label, "%s", values[i].name);
         if (values[i].tool_id) {
             sprintf(action_id, "tool_set_%s", values[i].tool_id);
@@ -512,28 +525,28 @@ static void tools_panel(void)
         if (gui_selectable_icon(label, &v, values[i].icon)) {
             action_exec(action, "");
         }
-        auto_grid(nb, i, 4);
+        auto_grid(nb, i, GUI_TOOLS_COLUMNS_NB);
     }
     gui_group_end();
     auto_adjust_panel_size();
 
     ImGui::SetNextTreeNodeOpen(true, ImGuiCond_Once);
     if (gui_collapsing_header("Tool Options"))
-        tool_gui(goxel->tool);
+        tool_gui(goxel.tool);
 }
 
-static void toggle_layer_only_visible(goxel_t *goxel, layer_t *layer)
+static void toggle_layer_only_visible(layer_t *layer)
 {
     layer_t *other;
     bool others_all_invisible = true;
-    DL_FOREACH(goxel->image->layers, other) {
+    DL_FOREACH(goxel.image->layers, other) {
         if (other == layer) continue;
         if (other->visible) {
             others_all_invisible = false;
             break;
         }
     }
-    DL_FOREACH(goxel->image->layers, other)
+    DL_FOREACH(goxel.image->layers, other)
         other->visible = others_all_invisible;
     layer->visible = true;
 }
@@ -614,20 +627,20 @@ static void layers_panel(void)
     int i = 0, icon, bbox[2][3];
     bool current, visible, bounded;
     gui_group_begin(NULL);
-    DL_FOREACH(goxel->image->layers, layer) {
-        current = goxel->image->active_layer == layer;
+    DL_FOREACH(goxel.image->layers, layer) {
+        current = goxel.image->active_layer == layer;
         visible = layer->visible;
         icon = layer->base_id ? ICON_LINK : -1;
         layer_item(i, icon, &visible, &current,
                    layer->name, sizeof(layer->name));
-        if (current && goxel->image->active_layer != layer) {
-            goxel->image->active_layer = layer;
+        if (current && goxel.image->active_layer != layer) {
+            goxel.image->active_layer = layer;
             goxel_update_meshes(-1);
         }
         if (visible != layer->visible) {
             layer->visible = visible;
             if (ImGui::IsKeyDown(KEY_LEFT_SHIFT))
-                toggle_layer_only_visible(goxel, layer);
+                toggle_layer_only_visible(layer);
             goxel_update_meshes(-1);
         }
         i++;
@@ -648,13 +661,18 @@ static void layers_panel(void)
     gui_action_button("img_clone_layer", "Clone", 1, "");
     gui_action_button("img_merge_visible_layers", "Merge visible", 1, "");
 
-    layer = goxel->image->active_layer;
+    layer = goxel.image->active_layer;
     bounded = !box_is_null(layer->box);
     if (bounded && gui_button("Crop to box", 1, 0)) {
         mesh_crop(layer->mesh, layer->box);
         goxel_update_meshes(-1);
     }
+    if (!box_is_null(goxel.image->box) && gui_button("Crop to image", 1, 0)) {
+        mesh_crop(layer->mesh, goxel.image->box);
+        goxel_update_meshes(-1);
+    }
     gui_group_end();
+    auto_adjust_panel_size();
 
     if (layer->base_id) {
         gui_group_begin(NULL);
@@ -662,6 +680,9 @@ static void layers_panel(void)
         gui_action_button("img_select_parent_layer", "Select parent", 1, "");
         gui_group_end();
     }
+    if (layer->image) {
+        gui_action_button("img_image_layer_to_mesh", "To Mesh", 1, "");
+    }
     if (ImGui::Checkbox("Bounded", &bounded)) {
         if (bounded) {
             mesh_get_bbox(layer->mesh, bbox, true);
@@ -681,17 +702,17 @@ static void view_panel(void)
         uint8_t    *color;
         const char *label;
     } COLORS[] = {
-        {goxel->back_color, "Back color"},
-        {goxel->grid_color, "Grid color"},
-        {goxel->image_box_color, "Box color"},
+        {goxel.back_color, "Back color"},
+        {goxel.grid_color, "Grid color"},
+        {goxel.image_box_color, "Box color"},
     };
     int i;
     ImVec4 c;
 
     gui_group_begin("Light");
-    gui_angle("Pitch", &goxel->rend.light.pitch, -90, +90);
-    gui_angle("Yaw", &goxel->rend.light.yaw, 0, 360);
-    gui_checkbox("Fixed", &goxel->rend.light.fixed, NULL);
+    gui_angle("Pitch", &goxel.rend.light.pitch, -90, +90);
+    gui_angle("Yaw", &goxel.rend.light.yaw, 0, 360);
+    gui_checkbox("Fixed", &goxel.rend.light.fixed, NULL);
     gui_group_end();
 
     for (i = 0; i < (int)ARRAY_SIZE(COLORS); i++) {
@@ -708,6 +729,7 @@ static void view_panel(void)
         ImGui::Text("%s", COLORS[i].label);
         ImGui::PopID();
     }
+    gui_checkbox("Hide box", &goxel.hide_box, NULL);
 
 }
 
@@ -726,7 +748,10 @@ static bool render_palette_entry(const uint8_t color[4], uint8_t target[4])
                            0xFFFFFFFF, 0, 0, 1);
     }
     ImGui::PopStyleColor(2);
-    if (ret) memcpy(target, color, 4);
+    if (ret) {
+        on_click();
+        memcpy(target, color, 4);
+    }
     return ret;
 }
 
@@ -734,30 +759,30 @@ static bool render_palette_entry(const uint8_t color[4], uint8_t target[4])
 static void palette_panel(void)
 {
     palette_t *p;
-    int i, current, nb = 0, nb_col = 8;
+    int i, current, nb = 0, nb_col = GUI_PALETTE_COLUMNS_NB;
     const char **names;
 
-    DL_COUNT(goxel->palettes, p, nb);
+    DL_COUNT(goxel.palettes, p, nb);
     names = (const char**)calloc(nb, sizeof(*names));
 
     i = 0;
-    DL_FOREACH(goxel->palettes, p) {
-        if (p == goxel->palette) current = i;
+    DL_FOREACH(goxel.palettes, p) {
+        if (p == goxel.palette) current = i;
         names[i++] = p->name;
     }
     ImGui::PushItemWidth(-1);
     if (gui_combo("##palettes", &current, names, nb)) {
-        goxel->palette = goxel->palettes;
-        for (i = 0; i < current; i++) goxel->palette = goxel->palette->next;
+        goxel.palette = goxel.palettes;
+        for (i = 0; i < current; i++) goxel.palette = goxel.palette->next;
     }
     ImGui::PopItemWidth();
     free(names);
 
-    p = goxel->palette;
+    p = goxel.palette;
 
     for (i = 0; i < p->size; i++) {
         ImGui::PushID(i);
-        render_palette_entry(p->entries[i].color, goxel->painter.color);
+        render_palette_entry(p->entries[i].color, goxel.painter.color);
         auto_adjust_panel_size();
         if ((i + 1) % nb_col && i != p->size - 1) ImGui::SameLine();
         ImGui::PopID();
@@ -771,17 +796,18 @@ static void material_advanced_panel(void)
 
     ImGui::PushID("render_advanced");
     gui_group_begin(NULL);
-    v = goxel->rend.settings.border_shadow;
+    v = goxel.rend.settings.border_shadow;
     if (gui_input_float("bshadow", &v, 0.1, 0.0, 1.0, NULL)) {
         v = clamp(v, 0, 1); \
-        goxel->rend.settings.border_shadow = v;
+        goxel.rend.settings.border_shadow = v;
     }
 #define MAT_FLOAT(name, min, max) \
-    v = goxel->rend.settings.name;  \
+    v = goxel.rend.settings.name;  \
     if (gui_input_float(#name, &v, 0.1, min, max, NULL)) { \
         v = clamp(v, min, max); \
-        goxel->rend.settings.name = v; \
-    }
+        goxel.rend.settings.name = v; \
+    } \
+    auto_adjust_panel_size();
 
     MAT_FLOAT(ambient, 0, 1);
     MAT_FLOAT(diffuse, 0, 1);
@@ -793,18 +819,19 @@ static void material_advanced_panel(void)
     gui_group_end();
 
     ImGui::CheckboxFlags("Borders",
-            (unsigned int*)&goxel->rend.settings.effects, EFFECT_BORDERS);
+            (unsigned int*)&goxel.rend.settings.effects, EFFECT_BORDERS);
     ImGui::CheckboxFlags("Borders all",
-            (unsigned int*)&goxel->rend.settings.effects, EFFECT_BORDERS_ALL);
+            (unsigned int*)&goxel.rend.settings.effects, EFFECT_BORDERS_ALL);
     ImGui::CheckboxFlags("See back",
-            (unsigned int*)&goxel->rend.settings.effects, EFFECT_SEE_BACK);
+            (unsigned int*)&goxel.rend.settings.effects, EFFECT_SEE_BACK);
     if (ImGui::CheckboxFlags("Marching Cubes",
-            (unsigned int*)&goxel->rend.settings.effects, EFFECT_MARCHING_CUBES)) {
-        goxel->rend.settings.smoothness = 1;
+            (unsigned int*)&goxel.rend.settings.effects, EFFECT_MARCHING_CUBES)) {
+        goxel.rend.settings.smoothness = 1;
     }
-    if (goxel->rend.settings.effects & EFFECT_MARCHING_CUBES)
+    auto_adjust_panel_size();
+    if (goxel.rend.settings.effects & EFFECT_MARCHING_CUBES)
         ImGui::CheckboxFlags("Flat",
-            (unsigned int*)&goxel->rend.settings.effects, EFFECT_FLAT);
+            (unsigned int*)&goxel.rend.settings.effects, EFFECT_FLAT);
 
     ImGui::PopID();
 }
@@ -823,20 +850,22 @@ static void material_panel(void)
     for (i = 0; i < nb; i++) {
         render_get_default_settings(i, &name, &settings);
         names[i] = name;
-        if (memcmp(&goxel->rend.settings, &settings,
+        if (memcmp(&goxel.rend.settings, &settings,
                          sizeof(settings)) == 0)
             current = i;
     }
     gui_text("Presets:");
     if (gui_combo("##Presets", &current, names, nb)) {
         render_get_default_settings(current, NULL, &settings);
-        goxel->rend.settings = settings;
+        goxel.rend.settings = settings;
     }
     free(names);
 
-    v = goxel->rend.settings.shadow;
-    if (gui_input_float("shadow", &v, 0.1, 0, 0, NULL)) {
-        goxel->rend.settings.shadow = clamp(v, 0, 1);
+    if (!DEFINED(GOXEL_NO_SHADOW)) {
+        v = goxel.rend.settings.shadow;
+        if (gui_input_float("shadow", &v, 0.1, 0, 0, NULL)) {
+            goxel.rend.settings.shadow = clamp(v, 0, 1);
+        }
     }
     ImGui::SetNextTreeNodeOpen(false, ImGuiCond_Once);
     if (gui_collapsing_header("Render Advanced"))
@@ -848,33 +877,33 @@ static void render_panel(void)
     int i;
     int maxsize;
     const char *path;
-    typeof(goxel->render_task) *task = &goxel->render_task;
+    typeof(goxel.render_task) *task = &goxel.render_task;
 
-    goxel->no_edit = task->status || gui->popup.title;
+    goxel.no_edit = task->status || gui->popup_count;
 
     GL(glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxsize));
     maxsize /= 2; // Because png export already double it.
-    goxel->show_export_viewport = true;
+    goxel.show_export_viewport = true;
     gui_group_begin(NULL);
-    i = goxel->image->export_width;
+    i = goxel.image->export_width;
     if (gui_input_int("w", &i, 1, maxsize))
-        goxel->image->export_width = clamp(i, 1, maxsize);
-    i = goxel->image->export_height;
+        goxel.image->export_width = clamp(i, 1, maxsize);
+    i = goxel.image->export_height;
     if (gui_input_int("h", &i, 1, maxsize))
-        goxel->image->export_height = clamp(i, 1, maxsize);
+        goxel.image->export_height = clamp(i, 1, maxsize);
     if (gui_button("Fit screen", 1, 0)) {
-        goxel->image->export_width = gui->view.rect[2];
-        goxel->image->export_height = gui->view.rect[3];
+        goxel.image->export_width = gui->view.rect[2];
+        goxel.image->export_height = gui->view.rect[3];
     }
     if (gui_button("Set output", 1, 0)) {
         path = noc_file_dialog_open(NOC_FILE_DIALOG_SAVE, "png\0*.png\0", NULL,
                                     "untitled.png");
-        if (path) strcpy(goxel->render_task.output, path);
+        if (path) strcpy(goxel.render_task.output, path);
     }
     gui_group_end();
 
-    if (*goxel->render_task.output)
-        gui_text("%s", goxel->render_task.output);
+    if (*goxel.render_task.output)
+        gui_text("%s", goxel.render_task.output);
 
     if (task->status == 0 && gui_button("Render", 0, 0)) task->status = 1;
     if (task->status == 1 && gui_button("Cancel", 0, 0)) task->status = 0;
@@ -884,8 +913,8 @@ static void render_panel(void)
         task->force_restart = true;
     }
 
-    if (goxel->render_task.status) {
-        gui_text("%d/100", (int)(goxel->render_task.progress * 100));
+    if (goxel.render_task.status) {
+        gui_text("%d/100", (int)(goxel.render_task.progress * 100));
     }
 
 }
@@ -893,21 +922,23 @@ static void render_panel(void)
 static void image_panel(void)
 {
     bool bounded;
-    image_t *image = goxel->image;
+    image_t *image = goxel.image;
     int bbox[2][3];
     float (*box)[4][4] = &image->box;
 
     bounded = !box_is_null(*box);
     if (ImGui::Checkbox("Bounded", &bounded)) {
         if (bounded) {
-            mesh_get_bbox(goxel->layers_mesh, bbox, true);
+            mesh_get_bbox(goxel.layers_mesh, bbox, true);
             if (bbox[0][0] > bbox[1][0]) memset(bbox, 0, sizeof(bbox));
             bbox_from_aabb(*box, bbox);
         } else {
             mat4_copy(mat4_zero, *box);
+            goxel.snap_mask |= SNAP_PLANE;
         }
     }
     if (bounded) gui_bbox(*box);
+    auto_adjust_panel_size();
 }
 
 static void cameras_panel(void)
@@ -916,17 +947,18 @@ static void cameras_panel(void)
     int i = 0;
     bool current;
     gui_group_begin(NULL);
-    DL_FOREACH(goxel->image->cameras, cam) {
-        current = goxel->image->active_camera == cam;
+    DL_FOREACH(goxel.image->cameras, cam) {
+        current = goxel.image->active_camera == cam;
         if (layer_item(i, -1, NULL, &current, cam->name, sizeof(cam->name))) {
             if (current) {
-                camera_set(&goxel->camera, cam);
-                goxel->image->active_camera = cam;
+                camera_set(&goxel.camera, cam);
+                goxel.image->active_camera = cam;
             } else {
-                goxel->image->active_camera = NULL;
+                goxel.image->active_camera = NULL;
             }
         }
         i++;
+        auto_adjust_panel_size();
     }
     gui_group_end();
     gui_action_button("img_new_camera", NULL, 0, "");
@@ -938,7 +970,7 @@ static void cameras_panel(void)
     gui_action_button("img_move_camera_down", NULL, 0, "");
     auto_adjust_panel_size();
 
-    cam = &goxel->camera;
+    cam = &goxel.camera;
     gui_input_float("dist", &cam->dist, 10.0, 0, 0, NULL);
 
     gui_group_begin("Offset");
@@ -961,10 +993,12 @@ static void cameras_panel(void)
 
 static void debug_panel(void)
 {
-    ImGui::Text("FPS: %d", (int)round(goxel->fps));
+    ImGui::Text("FPS: %d", (int)round(goxel.fps));
+    if (!DEFINED(GLES2))
+        gui_checkbox("Show wireframe", &goxel.show_wireframe, NULL);
 }
 
-static void import_image_plane(goxel_t *goxel)
+static void import_image_plane(void)
 {
     const char *path;
     path = noc_file_dialog_open(NOC_FILE_DIALOG_OPEN,
@@ -978,7 +1012,7 @@ static bool shift_alpha_popup(void *data)
     static int v = 0;
     static mesh_t *original_mesh = NULL;
     mesh_t *mesh;
-    mesh = goxel->image->active_layer->mesh;
+    mesh = goxel.image->active_layer->mesh;
     if (!original_mesh)
         original_mesh = mesh_copy(mesh);
     if (ImGui::InputInt("shift", &v, 1)) {
@@ -1024,6 +1058,12 @@ static bool about_popup(void *data)
     return gui_button("OK", 0, 0);
 }
 
+static bool alert_popup(void *data)
+{
+    if (data) gui_text((const char *)data);
+    return gui_button("OK", 0, 0);
+}
+
 static int check_action_shortcut(action_t *action, void *user)
 {
     ImGuiIO& io = ImGui::GetIO();
@@ -1063,11 +1103,11 @@ static int export_menu_action_callback(action_t *a, void *user)
     return 0;
 }
 
-static bool render_menu_item(const char *id, const char *label)
+static bool render_menu_item(const char *id, const char *label, bool enabled)
 {
     const action_t *action = action_get(id);
     assert(action);
-    if (ImGui::MenuItem(label, action->shortcut)) {
+    if (ImGui::MenuItem(label, action->shortcut, false, enabled)) {
         action_exec(action, "");
         return true;
     }
@@ -1081,11 +1121,12 @@ static void render_menu(void)
 
     if (!ImGui::BeginMenuBar()) return;
     if (ImGui::BeginMenu("File")) {
-        render_menu_item("save", "Save");
-        render_menu_item("save_as", "Save as");
-        render_menu_item("open", "Open");
+        render_menu_item("save", "Save",
+                image_get_key(goxel.image) != goxel.image->saved_key);
+        render_menu_item("save_as", "Save as", true);
+        render_menu_item("open", "Open", true);
         if (ImGui::BeginMenu("Import...")) {
-            if (ImGui::MenuItem("image plane")) import_image_plane(goxel);
+            if (ImGui::MenuItem("image plane")) import_image_plane();
             actions_iter(import_menu_action_callback, NULL);
             ImGui::EndMenu();
         }
@@ -1093,16 +1134,16 @@ static void render_menu(void)
             actions_iter(export_menu_action_callback, NULL);
             ImGui::EndMenu();
         }
-        render_menu_item("quit", "Quit");
+        render_menu_item("quit", "Quit", true);
         ImGui::EndMenu();
     }
     if (ImGui::BeginMenu("Edit")) {
         if (ImGui::MenuItem("Clear", "Delete"))
             action_exec2("layer_clear", "");
-        render_menu_item("undo", "Undo");
-        render_menu_item("redo", "Redo");
-        render_menu_item("copy", "Copy");
-        render_menu_item("past", "Past");
+        render_menu_item("undo", "Undo", true);
+        render_menu_item("redo", "Redo", true);
+        render_menu_item("copy", "Copy", true);
+        render_menu_item("past", "Past", true);
         if (ImGui::MenuItem("Shift Alpha"))
             gui_open_popup("Shift Alpha", 0, NULL, shift_alpha_popup);
         if (ImGui::MenuItem("Settings"))
@@ -1111,11 +1152,11 @@ static void render_menu(void)
         ImGui::EndMenu();
     }
     if (ImGui::BeginMenu("View")) {
-        render_menu_item("view_left", "Left");
-        render_menu_item("view_right", "Right");
-        render_menu_item("view_front", "Front");
-        render_menu_item("view_top", "Top");
-        render_menu_item("view_default", "Default");
+        render_menu_item("view_left", "Left", true);
+        render_menu_item("view_right", "Right", true);
+        render_menu_item("view_front", "Front", true);
+        render_menu_item("view_top", "Top", true);
+        render_menu_item("view_default", "Default", true);
         ImGui::EndMenu();
     }
     if (ImGui::BeginMenu("Help")) {
@@ -1176,9 +1217,9 @@ static int render_mode_select(void)
     };
     gui_group_begin(NULL);
     for (i = 0; i < (int)ARRAY_SIZE(values); i++) {
-        v = goxel->painter.mode == values[i].mode;
+        v = goxel.painter.mode == values[i].mode;
         if (gui_selectable_icon(values[i].name, &v, values[i].icon)) {
-            goxel->painter.mode = values[i].mode;
+            goxel.painter.mode = values[i].mode;
         }
         auto_grid(ARRAY_SIZE(values), i, 4);
     }
@@ -1197,7 +1238,7 @@ static void render_top_bar(void)
     gui_same_line();
     render_mode_select();
     gui_same_line();
-    gui_color("##color", goxel->painter.color);
+    gui_color("##color", goxel.painter.color);
 }
 
 static void render_left_panel(void)
@@ -1251,7 +1292,7 @@ static void render_left_panel(void)
     if (gui->current_panel) {
         ImGui::SameLine();
         ImGui::BeginGroup();
-        goxel->show_export_viewport = false;
+        goxel.show_export_viewport = false;
         ImGui::PushID("panel");
         ImGui::PushID(PANELS[gui->current_panel].name);
         PANELS[gui->current_panel].fn();
@@ -1262,6 +1303,46 @@ static void render_left_panel(void)
     ImGui::EndChild();
 }
 
+static void render_popups(int index)
+{
+    typeof(gui->popup[0]) *popup;
+    ImGuiIO& io = ImGui::GetIO();
+
+    popup = &gui->popup[index];
+    if (!popup->title) return;
+
+    if (!popup->opened) {
+        ImGui::OpenPopup(popup->title);
+        popup->opened = true;
+    }
+    int flags = ImGuiWindowFlags_AlwaysAutoResize | ImGuiWindowFlags_NoMove;
+    if (popup->flags & GUI_POPUP_FULL) {
+        ImGui::SetNextWindowSize(ImVec2(io.DisplaySize.x - 40,
+                                        io.DisplaySize.y - 40),
+                (popup->flags & GUI_POPUP_RESIZE) ?  ImGuiCond_Once : 0);
+    }
+    if (popup->flags & GUI_POPUP_RESIZE) {
+        flags &= ~(ImGuiWindowFlags_NoMove |
+                   ImGuiWindowFlags_AlwaysAutoResize);
+    }
+    if (ImGui::BeginPopupModal(popup->title, NULL, flags)) {
+        typeof(popup->func) func = popup->func;
+        if (func(popup->data)) {
+            ImGui::CloseCurrentPopup();
+            gui->popup_count--;
+            popup->title = NULL;
+            popup->func = NULL;
+            free(popup->data);
+            popup->data = NULL;
+            if (popup->on_closed) popup->on_closed();
+            popup->on_closed = NULL;
+            popup->opened = false;
+        }
+        render_popups(index + 1);
+        ImGui::EndPopup();
+    }
+}
+
 void gui_iter(const inputs_t *inputs)
 {
     if (!gui) gui_init(inputs);
@@ -1272,26 +1353,29 @@ void gui_iter(const inputs_t *inputs)
     const theme_t *theme = theme_get();
     gesture_t *gestures[] = {&gui->gestures.drag, &gui->gestures.hover};
     float display_rect[4] = {
-        0.f, 0.f, (float)goxel->screen_size[0], (float)goxel->screen_size[1]};
+        0.f, 0.f, (float)goxel.screen_size[0], (float)goxel.screen_size[1]};
     float font_size = ImGui::GetFontSize();
 
-    io.DisplaySize = ImVec2((float)goxel->screen_size[0],
-                            (float)goxel->screen_size[1]);
+    io.DisplaySize = ImVec2((float)goxel.screen_size[0],
+                            (float)goxel.screen_size[1]);
 
-    io.DisplayFramebufferScale = ImVec2(goxel->screen_scale,
-                                        goxel->screen_scale);
+    io.DisplayFramebufferScale = ImVec2(goxel.screen_scale,
+                                        goxel.screen_scale);
     io.DeltaTime = 1.0 / 60;
-    gesture_update(2, gestures, inputs, display_rect, gui);
-    io.MouseWheel = inputs->mouse_wheel;
+    if (inputs) {
+        gesture_update(2, gestures, inputs, display_rect, gui);
+        io.MouseWheel = inputs->mouse_wheel;
 
-    for (i = 0; i < ARRAY_SIZE(inputs->keys); i++)
-        io.KeysDown[i] = inputs->keys[i];
-    io.KeyShift = inputs->keys[KEY_LEFT_SHIFT] ||
-                  inputs->keys[KEY_RIGHT_SHIFT];
-    io.KeyCtrl = inputs->keys[KEY_CONTROL];
-    for (i = 0; i < ARRAY_SIZE(inputs->chars); i++) {
-        if (!inputs->chars[i]) break;
-        io.AddInputCharacter(inputs->chars[i]);
+        for (i = 0; i < ARRAY_SIZE(inputs->keys); i++)
+            io.KeysDown[i] = inputs->keys[i];
+        io.KeyShift = inputs->keys[KEY_LEFT_SHIFT] ||
+                      inputs->keys[KEY_RIGHT_SHIFT];
+        io.KeyCtrl = inputs->keys[KEY_CONTROL];
+        for (i = 0; i < ARRAY_SIZE(inputs->chars); i++) {
+            if (!inputs->chars[i]) break;
+            io.AddInputCharacter(inputs->chars[i]);
+        }
+        memset((void*)inputs->chars, 0, sizeof(inputs->chars));
     }
 
     // Setup theme.
@@ -1334,41 +1418,12 @@ void gui_iter(const inputs_t *inputs)
     render_menu();
     render_top_bar();
 
-    goxel->no_edit = false; // Set depending on what panel is selected.
-    goxel->use_cycles = false;  // Also set depending on the panel.
+    goxel.no_edit = false; // Set depending on what panel is selected.
+    goxel.use_cycles = false;  // Also set depending on the panel.
     render_left_panel();
     ImGui::SameLine();
 
-    if (gui->popup.title) {
-        if (!gui->popup.opened) {
-            ImGui::OpenPopup(gui->popup.title);
-            gui->popup.opened = true;
-        }
-        int flags = ImGuiWindowFlags_AlwaysAutoResize |
-                    ImGuiWindowFlags_NoMove;
-        if (gui->popup.flags & GUI_POPUP_FULL) {
-            ImGui::SetNextWindowSize(ImVec2(io.DisplaySize.x - 40,
-                                            io.DisplaySize.y - 40),
-                        (gui->popup.flags & GUI_POPUP_RESIZE) ?
-                            ImGuiCond_Once : 0);
-        }
-        if (gui->popup.flags & GUI_POPUP_RESIZE) {
-            flags &= ~(ImGuiWindowFlags_NoMove |
-                       ImGuiWindowFlags_AlwaysAutoResize);
-        }
-        if (ImGui::BeginPopupModal(gui->popup.title, NULL, flags)) {
-            typeof(gui->popup.func) func = gui->popup.func;
-            if (func(gui->popup.data)) {
-                ImGui::CloseCurrentPopup();
-                // Only reset if there was no change.
-                if (func == gui->popup.func) {
-                    free(gui->popup.data);
-                    memset(&gui->popup, 0, sizeof(gui->popup));
-                }
-            }
-            ImGui::EndPopup();
-        }
-    }
+    render_popups(0);
 
     ImGui::BeginChild("3d view", ImVec2(0, 0), false,
                       ImGuiWindowFlags_NoScrollWithMouse);
@@ -1388,20 +1443,21 @@ void gui_iter(const inputs_t *inputs)
                           io.DisplaySize.y - (canvas_pos.y + canvas_size.y),
                           canvas_size.x, canvas_size.y};
     // Call mouse_in_view with inputs in the view referential.
-    if (!(!gui->mouse_in_view && inputs->mouse_wheel) &&
+    if (    inputs &&
+            !(!gui->mouse_in_view && inputs->mouse_wheel) &&
             !gui->capture_mouse) {
         inputs_t inputs2 = *inputs;
         for (i = 0; i < ARRAY_SIZE(inputs->touches); i++) {
             inputs2.touches[i].pos[1] =
                 io.DisplaySize.y - inputs2.touches[i].pos[1];
         }
-        goxel_mouse_in_view(view_rect, &inputs2);
+        goxel_mouse_in_view(view_rect, &inputs2, !io.WantCaptureKeyboard);
     }
 
-    render_axis_arrows(goxel, view_size);
-    ImGui::Text("%s", goxel->hint_text ?: "");
+    render_axis_arrows(view_size);
+    ImGui::Text("%s", goxel.hint_text ?: "");
     ImGui::SameLine(180);
-    ImGui::Text("%s", goxel->help_text ?: "");
+    ImGui::Text("%s", goxel.help_text ?: "");
 
     ImGui::EndChild();
     ImGui::End();
@@ -1409,30 +1465,22 @@ void gui_iter(const inputs_t *inputs)
     // Handle the shortcuts.  XXX: this should be done with actions.
     if (ImGui::IsKeyPressed(KEY_DELETE, false))
         action_exec2("layer_clear", "");
-    if (ImGui::IsKeyPressed(' ', false) && goxel->painter.mode == MODE_OVER)
-        goxel->painter.mode = MODE_SUB;
-    if (ImGui::IsKeyReleased(' ') && goxel->painter.mode == MODE_SUB)
-        goxel->painter.mode = MODE_OVER;
-    if (ImGui::IsKeyReleased(' ') && goxel->painter.mode == MODE_SUB)
-        goxel->painter.mode = MODE_OVER;
-    float last_tool_radius = goxel->tool_radius;
 
     if (!io.WantCaptureKeyboard) {
-        if (isCharPressed('[')) goxel->tool_radius -= 0.5;
-        if (isCharPressed(']')) goxel->tool_radius += 0.5;
-        if (goxel->tool_radius != last_tool_radius) {
-            goxel->tool_radius = clamp(goxel->tool_radius, 0.5, 64);
-        }
 
-        // XXX: this won't map correctly to a French keyboard.  Unfortunately as
-        // far as I can tell, GLFW3 does not allow to check for ctrl-Z on any
-        // layout on Windows.  For the moment I just ignore the problem until I
-        // either find a solution, either find a replacement for GLFW.
-        // With the GLUT backend ctrl-z and ctrl-y are actually reported as the
-        // key 25 and 26, which might makes more sense.  Here I test for both.
-        if (isCharPressed(26)) action_exec2("undo", "");
-        if (isCharPressed(25)) action_exec2("redo", "");
-        // Check the action shortcuts.
+        // Handle space key toggle over/sub modes.
+        if (    ImGui::IsKeyPressed(' ', false) &&
+                goxel.painter.mode == MODE_OVER)
+            goxel.painter.mode = MODE_SUB;
+        if (ImGui::IsKeyReleased(' ') && goxel.painter.mode == MODE_SUB)
+            goxel.painter.mode = MODE_OVER;
+
+        float last_tool_radius = goxel.tool_radius;
+        if (isCharPressed('[')) goxel.tool_radius -= 0.5;
+        if (isCharPressed(']')) goxel.tool_radius += 0.5;
+        if (goxel.tool_radius != last_tool_radius) {
+            goxel.tool_radius = clamp(goxel.tool_radius, 0.5, 64);
+        }
         actions_iter(check_action_shortcut, NULL);
     }
     ImGui::EndFrame();
@@ -1530,6 +1578,7 @@ bool gui_input_int(const char *label, int *v, int minv, int maxv)
 bool gui_input_float(const char *label, float *v, float step,
                      float minv, float maxv, const char *format)
 {
+    const theme_t *theme = theme_get();
     bool self_group = false, ret;
     if (minv == 0.f && maxv == 0.f) {
         minv = -FLT_MAX;
@@ -1546,6 +1595,11 @@ bool gui_input_float(const char *label, float *v, float step,
     ret = ImGui::GoxInputFloat(label, v, step, minv, maxv, format);
     if (ret) on_click();
     if (self_group) gui_group_end();
+
+    // Empirical panel size required to render this element.
+    auto_adjust_panel_size(
+            ImGui::CalcTextSize(label).x +
+            theme->sizes.item_height * 4);
     return ret;
 }
 
@@ -1797,8 +1851,6 @@ bool gui_button(const char *label, float size, int icon)
     if (icon) {
         center = GetItemRectMin() + ImVec2(GetItemRectSize().y / 2,
                                            GetItemRectSize().y / 2);
-        center.x += 0.5;
-        center.y += 0.5;
         uv0 = ImVec2(((icon - 1) % 8) / 8.0, ((icon - 1) / 8) / 8.0);
         uv1 = ImVec2(uv0.x + 1. / 8, uv0.y + 1. / 8);
         draw_list->AddImage((void*)(intptr_t)g_tex_icons->tex,
@@ -1810,6 +1862,19 @@ bool gui_button(const char *label, float size, int icon)
     return ret;
 }
 
+bool gui_button_right(const char *label, int icon)
+{
+    const theme_t *theme = theme_get();
+    float text_size = ImGui::CalcTextSize(label).x;
+    float w = text_size + 2 * theme->sizes.item_padding_h;
+    w = max(w, theme->sizes.item_height);
+    w += theme->sizes.item_padding_h;
+    gui_same_line();
+    ImGui::Dummy(ImVec2(ImGui::GetContentRegionAvailWidth() - w, 0));
+    gui_same_line();
+    return gui_button(label, 0, icon);
+}
+
 bool gui_input_text(const char *label, char *txt, int size)
 {
     return InputText(label, txt, size);
@@ -1903,11 +1968,18 @@ bool gui_quat(const char *label, float q[4])
 void gui_open_popup(const char *title, int flags, void *data,
                     bool (*func)(void *data))
 {
-    gui->popup.title = title;
-    gui->popup.func = func;
-    gui->popup.flags = flags;
-    assert(!gui->popup.data);
-    gui->popup.data = data;
+    typeof(gui->popup[0]) *popup;
+    popup = &gui->popup[gui->popup_count++];
+    popup->title = title;
+    popup->func = func;
+    popup->flags = flags;
+    assert(!popup->data);
+    popup->data = data;
+}
+
+void gui_on_popup_closed(void (*func)(void))
+{
+    gui->popup[gui->popup_count - 1].on_closed = func;
 }
 
 void gui_popup_body_begin(void) {
@@ -1918,12 +1990,6 @@ void gui_popup_body_end(void) {
     ImGui::EndChild();
 }
 
-static bool alert_popup(void *data)
-{
-    if (data) gui_text((const char *)data);
-    return gui_button("OK", 0, 0);
-}
-
 void gui_alert(const char *title, const char *msg)
 {
     gui_open_popup(title, 0, msg ? strdup(msg) : NULL, alert_popup);
@@ -1959,4 +2025,16 @@ void gui_pop_id(void)
     ImGui::PopID();
 }
 
+void gui_floating_icon(int icon)
+{
+    ImDrawList* draw_list = ImGui::GetWindowDrawList();
+    ImVec2 uv0, uv1, pos;
+    uv0 = ImVec2(((icon - 1) % 8) / 8.0, ((icon - 1) / 8) / 8.0);
+    uv1 = ImVec2(uv0.x + 1. / 8, uv0.y + 1. / 8);
+    pos = GetItemRectMin();
+    draw_list->AddImage((void*)(intptr_t)g_tex_icons->tex,
+            pos, pos + ImVec2(32, 32),
+            uv0, uv1, get_icon_color(icon, 0));
+}
+
 }
