@@ -134,6 +134,17 @@ static void layer_delete(layer_t *layer)
     free(layer);
 }
 
+static uint64_t layer_get_key(const layer_t *layer)
+{
+    uint64_t key;
+    key = mesh_get_key(layer->mesh);
+    key = crc64(key, &layer->visible, sizeof(layer->visible));
+    key = crc64(key, &layer->name, sizeof(layer->name));
+    key = crc64(key, &layer->box, sizeof(layer->box));
+    key = crc64(key, &layer->mat, sizeof(layer->mat));
+    return key;
+}
+
 image_t *image_new(void)
 {
     layer_t *layer;
@@ -147,6 +158,8 @@ image_t *image_new(void)
     DL_APPEND(img->layers, layer);
     DL_APPEND2(img->history, img, history_prev, history_next);
     img->active_layer = layer;
+    // Prevent saving an empty image.
+    img->saved_key = image_get_key(img);
     return img;
 }
 
@@ -194,7 +207,7 @@ void image_delete(image_t *img)
 layer_t *image_add_layer(image_t *img)
 {
     layer_t *layer;
-    img = img ?: goxel->image;
+    img = img ?: goxel.image;
     layer = layer_new(img, "unnamed");
     layer->visible = true;
     DL_APPEND(img->layers, layer);
@@ -205,13 +218,13 @@ layer_t *image_add_layer(image_t *img)
 void image_delete_layer(image_t *img, layer_t *layer)
 {
     layer_t *other;
-    img = img ?: goxel->image;
+    img = img ?: goxel.image;
     layer = layer ?: img->active_layer;
     DL_DELETE(img->layers, layer);
     if (layer == img->active_layer) img->active_layer = NULL;
 
     // Unclone all layers cloned from this one.
-    DL_FOREACH(goxel->image->layers, other) {
+    DL_FOREACH(goxel.image->layers, other) {
         if (other->base_id == layer->id) {
             other->base_id = 0;
         }
@@ -230,7 +243,7 @@ void image_move_layer(image_t *img, layer_t *layer, int d)
 {
     assert(d == -1 || d == +1);
     layer_t *other = NULL;
-    img = img ?: goxel->image;
+    img = img ?: goxel.image;
     layer = layer ?: img->active_layer;
     if (d == -1) {
         other = layer->next;
@@ -256,7 +269,7 @@ static void image_move_layer_down(image_t *img, layer_t *layer)
 layer_t *image_duplicate_layer(image_t *img, layer_t *other)
 {
     layer_t *layer;
-    img = img ?: goxel->image;
+    img = img ?: goxel.image;
     other = other ?: img->active_layer;
     layer = layer_copy(other);
     layer->id = img_get_new_id(img);
@@ -269,7 +282,7 @@ layer_t *image_duplicate_layer(image_t *img, layer_t *other)
 layer_t *image_clone_layer(image_t *img, layer_t *other)
 {
     layer_t *layer;
-    img = img ?: goxel->image;
+    img = img ?: goxel.image;
     other = other ?: img->active_layer;
     layer = layer_clone(other);
     layer->id = img_get_new_id(img);
@@ -281,14 +294,14 @@ layer_t *image_clone_layer(image_t *img, layer_t *other)
 
 void image_unclone_layer(image_t *img, layer_t *layer)
 {
-    img = img ?: goxel->image;
+    img = img ?: goxel.image;
     layer = layer ?: img->active_layer;
     layer->base_id = 0;
 }
 
 void image_select_parent_layer(image_t *img, layer_t *layer)
 {
-    img = img ?: goxel->image;
+    img = img ?: goxel.image;
     layer = layer ?: img->active_layer;
     img->active_layer = img_get_layer(img, layer->base_id);
 }
@@ -296,7 +309,7 @@ void image_select_parent_layer(image_t *img, layer_t *layer)
 void image_merge_visible_layers(image_t *img)
 {
     layer_t *layer, *last = NULL;
-    img = img ?: goxel->image;
+    img = img ?: goxel.image;
     DL_FOREACH(img->layers, layer) {
         if (!layer->visible) continue;
         if (last) {
@@ -313,7 +326,7 @@ void image_merge_visible_layers(image_t *img)
 camera_t *image_add_camera(image_t *img)
 {
     camera_t *cam;
-    img = img ?: goxel->image;
+    img = img ?: goxel.image;
     cam = camera_new("unnamed");
     DL_APPEND(img->cameras, cam);
     img->active_camera = cam;
@@ -322,7 +335,7 @@ camera_t *image_add_camera(image_t *img)
 
 static void image_delete_camera(image_t *img, camera_t *cam)
 {
-    img = img ?: goxel->image;
+    img = img ?: goxel.image;
     cam = cam ?: img->active_camera;
     if (!cam) return;
     DL_DELETE(img->cameras, cam);
@@ -335,7 +348,7 @@ void image_move_camera(image_t *img, camera_t *cam, int d)
     // XXX: make a generic algo to move objects in a list.
     assert(d == -1 || d == +1);
     camera_t *other = NULL;
-    img = img ?: goxel->image;
+    img = img ?: goxel.image;
     cam = cam ?: img->active_camera;
     if (!cam) return;
     if (d == -1) {
@@ -452,7 +465,7 @@ void image_redo(image_t *img)
 void image_clear_layer(layer_t *layer, const float box[4][4])
 {
     painter_t painter;
-    layer = layer ?: goxel->image->active_layer;
+    layer = layer ?: goxel.image->active_layer;
     if (!box || box_is_null(box)) {
         mesh_clear(layer->mesh);
         return;
@@ -470,6 +483,59 @@ bool image_layer_can_edit(const image_t *img, const layer_t *layer)
     return !layer->base_id && !layer->image;
 }
 
+/*
+ * Function: image_get_key
+ * Return a value that is garantied to change when the image change.
+ */
+uint64_t image_get_key(const image_t *img)
+{
+    uint64_t key = 0, k;
+    layer_t *layer;
+    camera_t *camera;
+    DL_FOREACH(img->layers, layer) {
+        k = layer_get_key(layer);
+        key = crc64(key, &k, sizeof(k));
+    }
+    DL_FOREACH(img->cameras, camera) {
+        k = camera_get_key(camera);
+        key = crc64(key, &k, sizeof(k));
+    }
+    return key;
+}
+
+/*
+ * Turn an image layer into a mesh of 1 voxel depth.
+ */
+static void image_image_layer_to_mesh(image_t *img, layer_t *layer)
+{
+    uint8_t *data;
+    int i, j, w, h, bpp = 0, pos[3];
+    uint8_t c[4];
+    float p[3];
+    img = img ?: goxel.image;
+    layer = layer ?: img->active_layer;
+    mesh_accessor_t acc;
+
+    image_history_push(img);
+    data = img_read(layer->image->path, &w, &h, &bpp);
+    acc = mesh_get_accessor(layer->mesh);
+    for (j = 0; j < w; j++)
+    for (i = 0; i < h; i++) {
+        vec3_set(p, i / (float)h - 0.5, 0.5 - j / (float)w, 0);
+        mat4_mul_vec3(layer->mat, p, p);
+        pos[0] = round(p[0]);
+        pos[1] = round(p[1]);
+        pos[2] = round(p[2]);
+        memset(c, 0, 4);
+        c[3] = 255;
+        memcpy(c, data + (j * w + i) * bpp, bpp);
+        mesh_set_at(layer->mesh, &acc, pos, c);
+    }
+    texture_delete(layer->image);
+    layer->image = NULL;
+    free(data);
+}
+
 ACTION_REGISTER(layer_clear,
     .help = "Clear the current layer",
     .cfunc = image_clear_layer,
@@ -591,3 +657,10 @@ ACTION_REGISTER(img_move_camera_down,
     .flags = ACTION_TOUCH_IMAGE,
     .icon = ICON_ARROW_DOWNWARD,
 )
+
+ACTION_REGISTER(img_image_layer_to_mesh,
+    .help = "Turn an image layer into a mesh",
+    .cfunc = image_image_layer_to_mesh,
+    .csig = "vpp",
+    .flags = ACTION_TOUCH_IMAGE,
+)
