@@ -19,6 +19,10 @@
 
 #include "goxel.h"
 
+#ifndef RENDER_CACHE_SIZE
+#   define RENDER_CACHE_SIZE (1 * GB)
+#endif
+
 /*
  * The rendering is delayed from the time we call the different render
  * functions.  This allows to call `render_xxx` anywhere in the code, without
@@ -353,14 +357,16 @@ void render_init()
     GL(glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_index_buffer));
     GL(glBufferData(GL_ELEMENT_ARRAY_BUFFER, BATCH_QUAD_COUNT * 6 * 2,
                     index_array, GL_STATIC_DRAW));
+#ifndef GLES2
     GL(glEnable(GL_LINE_SMOOTH));
+#endif
 
     free(index_array);
     init_border_texture();
     init_bump_texture();
 
     // XXX: pick the proper memory size according to what is available.
-    g_items_cache = cache_create(1 * GB);
+    g_items_cache = cache_create(RENDER_CACHE_SIZE);
     g_cube_model = model3d_cube();
     g_line_model = model3d_line();
     g_wire_cube_model = model3d_wire_cube();
@@ -496,6 +502,20 @@ static void render_block_(renderer_t *rend, mesh_t *mesh,
     } else {
         GL(glDrawArrays(GL_TRIANGLES, 0, item->nb_elements * item->size));
     }
+
+#ifndef GLES2
+    if (effects & EFFECT_WIREFRAME) {
+        GL(glUniform1f(prog->u_m_amb_l, 0));
+        GL(glPolygonMode(GL_FRONT_AND_BACK, GL_LINE));
+        if (item->size == 4)
+            GL(glDrawElements(GL_TRIANGLES, item->nb_elements * 6,
+                              GL_UNSIGNED_SHORT, 0));
+        else
+            GL(glDrawArrays(GL_TRIANGLES, 0, item->nb_elements * item->size));
+        GL(glPolygonMode(GL_FRONT_AND_BACK, GL_FILL));
+        GL(glUniform1f(prog->u_m_amb_l, rend->settings.ambient));
+    }
+#endif
 }
 
 static void get_light_dir(const renderer_t *rend, bool model_view,
@@ -885,8 +905,10 @@ static void render_shadow_map(renderer_t *rend, float shadow_mvp[4][4])
     if (!g_shadow_map_fbo) {
         GL(glGenFramebuffers(1, &g_shadow_map_fbo));
         GL(glBindFramebuffer(GL_FRAMEBUFFER, g_shadow_map_fbo));
+        #ifndef GLES2
         GL(glDrawBuffer(GL_NONE));
         GL(glReadBuffer(GL_NONE));
+        #endif
         g_shadow_map = texture_new_surface(2048, 2048, 0);
         GL(glBindTexture(GL_TEXTURE_2D, g_shadow_map->tex));
         GL(glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT,
@@ -926,7 +948,7 @@ static void render_background(renderer_t *rend, const uint8_t col[4])
     vertex_t vertices[4];
     float c1[4], c2[4];
 
-    if (col[3] == 0) {
+    if (!col || col[3] == 0) {
         GL(glClearColor(0, 0, 0, 0));
         GL(glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));
         return;
@@ -981,7 +1003,7 @@ void render_submit(renderer_t *rend, const int rect[4],
     GL(glViewport(rect[0] * s, rect[1] * s, rect[2] * s, rect[3] * s));
     GL(glScissor(rect[0] * s, rect[1] * s, rect[2] * s, rect[3] * s));
     GL(glLineWidth(rend->scale));
-    if (clear_color) render_background(rend, clear_color);
+    render_background(rend, clear_color);
 
     DL_SORT(rend->items, item_sort_cmp);
     DL_FOREACH_SAFE(rend->items, item, tmp) {
@@ -1042,7 +1064,6 @@ int render_get_default_settings(int i, char **name, render_settings_t *out)
         case 4:
             if (name) *name = "Half smooth";
             out->smoothness = 0.2;
-            out->effects = EFFECT_BORDERS_ALL;
             break;
         case 5:
             if (name) *name = "Marching";
@@ -1050,5 +1071,6 @@ int render_get_default_settings(int i, char **name, render_settings_t *out)
             out->effects = EFFECT_MARCHING_CUBES | EFFECT_FLAT;
             break;
     }
+    if (DEFINED(GOXEL_NO_SHADOW)) out->shadow = 0;
     return 5;
 }
