@@ -65,7 +65,7 @@ static double get_log_time()
 void dolog(int level, const char *msg,
            const char *func, const char *file, int line, ...)
 {
-    const bool use_colors = !DEFINED(__APPLE__);
+    const bool use_colors = !DEFINED(__APPLE__) && !DEFINED(__EMSCRIPTEN__);
     char *msg_formatted, *full_msg;
     const char *format;
     char time_str[32] = "";
@@ -510,11 +510,11 @@ static const uint64_t crc64_tab[256] = {
  * Return:
  *   The new crc64 value.
  */
-uint64_t crc64(uint64_t crc, const uint8_t *s, uint64_t len)
+uint64_t crc64(uint64_t crc, const void *s, uint64_t len)
 {
     uint64_t j, byte;
     for (j = 0; j < len; j++) {
-        byte = s[j];
+        byte = ((const uint8_t*)s)[j];
         crc = crc64_tab[(uint8_t)crc ^ byte] ^ (crc >> 8);
     }
     return crc;
